@@ -32,12 +32,9 @@ class callbacks
     }
 
     /** @noinspection PhpUnused */
-    /**
-     * @throws \Exception
-     */
     public static function sendNewUserNotification(userEntity $user): userEntity
     {
-        if (ConfigSingleton::getInstance()->user->sendNewUserNotification ?? false) {
+        if (ConfigSingleton::getInstance()->user->sendNewUserNotification ?? true) {
             $notifier = System::makeRegisteredService(newUserNotifier::class, [
                 'user' => $user,
             ]);
