@@ -22,14 +22,12 @@ use Exception;
  * @property string $modified
  * @property string $modified_by
  * @property string $salt
- * @property string|array $roles
- * @method object getUserByEmail ($email)
- * @method userEntity load(string $id, bool $skipAccessCheck = false)
+ * @property string $roles
  * @mixin userOperator
  */
 class userEntity extends Entity
 {
-    public const TABLE = 'users';
+    final public const TABLE = 'users';
 
     /** @var userOperator */
     protected $operator;
@@ -52,45 +50,56 @@ class userEntity extends Entity
     }
 
     /**
-     * __call overload to allow for managing roles.
-     * @param $name
-     * @param $arguments
-     * @return $this
+     * override the magic load method, so we can intercept roles and un-sterilise them.
+     * @param $id
      * @throws Exception
      */
-    public function __call($name, $arguments){
-        switch($name) {
-            //modify load calls to include processed roles
-            case 'load':
-                if ($this->system->isInstalled()) {
-                    parent::__call($name, $arguments);
-                    //gives us an array of role id's for easy use later.
-                    if (!empty($this->data['roles']) && is_string($this->data['roles'])) {
-                        $this->data['roles'] = unserialize($this->data['roles']);
-                        $this->cacheRoles();
-                    }
-                    if (!empty($this->data['settings']) && is_string($this->data['settings'])) {
-                        $this->data['settings'] = json_decode($this->data['settings']);
-                    }
-                }
-                return $this;
-                //add permissions check to load multiple
-            case 'loadMultiple':
-                $currentUser = (currentUser::getInstance())::getCurrentUser();
-                if ($currentUser->isAllowed('ViewUsers')) {
-                    $arguments[0] = array_merge($arguments[0], ["id" => ["users.id", null, "IS NOT"]]);
-                }
-                break;
+    public function load($id, bool $skipAccessCheck = false): userEntity
+    {
+        if ($this->system->isInstalled()) {
+            parent::__call('load', [$id, $skipAccessCheck]);
+            //gives us an array of role id's for easy use later.
+            if (!empty($this->data['roles']) && is_string($this->data['roles'])) {
+                $this->data['roles'] = unserialize($this->data['roles']);
+                $this->cacheRoles();
+            }
         }
-        return parent::__call($name, $arguments);
+
+        return $this;
+    }
+
+    /**
+     * @param  array  $conditions
+     * @param  array  $orderBy
+     * @param  false  $limit
+     * @param  false  $offset
+     * @param  bool  $asEntities
+     * @return array
+     * @throws Exception
+     */
+    public function loadMultiple(
+        array $conditions = [],
+        array $orderBy = [],
+        $limit = false,
+        $offset = false,
+        bool $asEntities = true
+    ): array {
+        //we need to allow users with permission to bypass the default access control.
+        $currentUser = (currentUser::getInstance())::getCurrentUser();
+        if ($currentUser->isAllowed('ViewUsers')) {
+            $conditions = array_merge($conditions, ["id" => ["users.id", null, "IS NOT"]]);
+        }
+
+        return parent::loadMultiple($conditions, $orderBy, $limit, $offset, $asEntities);
     }
 
     protected function cacheRoles(): userEntity
     {
         foreach ($this->data['roles'] as $id) {
             if (!isset($this->cachedRoles[$id])) {
+                $roles = new roleEntity();
                 try {
-                    $this->cachedRoles[$id] = roleEntity::load($id, true);
+                    $this->cachedRoles[$id] = $roles->load($id, true);
                 } catch (Exception $e) {
                     user_error(
                         sprintf('Error caching roles: %s', $e->getMessage()),
@@ -115,7 +124,6 @@ class userEntity extends Entity
     /**
      * Function to add an ID to an array. PHP now throws "Indirect modification of overloaded property" when trying to set Array's using magic methods
      * @param $id
-     * @return $this
      */
     public function addRole($id): userEntity
     {
@@ -124,9 +132,6 @@ class userEntity extends Entity
         return $this;
     }
 
-    /**
-     * @return $this
-     */
     public function removeRoles(): userEntity
     {
         $this->data['roles'] = [];
@@ -143,22 +148,18 @@ class userEntity extends Entity
         $this->data['roles'] = $this->data['roles'] ?? [];
         //we need to revert the serialised data after storing, so it is still available as if we loaded the data
         if (isset($this->data['roles']) && !empty($this->data['roles']) && is_array($this->data['roles'])) {
-            $rolesCleanVersion = $this->data['roles'];
+            $cleanVersion = $this->data['roles'];
             $this->data['roles'] = serialize($this->data['roles']);
+            parent::store();
+            $this->data['roles'] = $cleanVersion;
+
+            return $this;
         }
         // if this is an array at this point, then its empty and has no values, so set it to null.
-        elseif (is_array($this->data['roles'])) {
+        if (is_array($this->data['roles'])) {
             $this->data['roles'] = null;
         }
-
-        if (isset($this->data['settings']) && !empty($this->data['settings'])) {
-            $settingsCleanVersion = $this->data['settings'];
-            $this->data['settings'] = json_encode($this->data['settings']);
-        }
-
         parent::store();
-        if (isset($rolesCleanVersion)) $this->data['roles'] = $rolesCleanVersion;
-        if (isset($settingsCleanVersion)) $this->data['settings'] = $settingsCleanVersion;
 
         return $this;
     }
@@ -202,11 +203,6 @@ class userEntity extends Entity
         return $this->id !== false ? sprintf('%s %s', $this->first_name, $this->last_name) : 'Missing User';
     }
 
-    public function getDetailedName(): string
-    {
-        return $this->id !== false ? sprintf('%s %s (%s)', $this->first_name, $this->last_name, $this->email) : 'Anonymous';
-    }
-
     public function isLoggedIn(): bool
     {
         return ($this->id !== null && !empty($this->id));
@@ -236,17 +232,11 @@ class userEntity extends Entity
                 'name' => "Last Name",
             ],
             "active" => [
-                'name' => "Active (Can Login)",
+                'name' => "Active",
                 'display' => function ($active) {
                     return $active ? 'Yes' : 'No';
                 },
             ],
-            "marked_fraudulent" => [
-                'name' => "Marked as Fradulent",
-                'display' => function ($markedFraudulent) {
-                    return $markedFraudulent ? 'Yes' : 'No';
-                },
-            ],
             "email" => [
                 'name' => "Email Address",
             ],
@@ -259,7 +249,7 @@ class userEntity extends Entity
             "last_login" => [
                 'name' => "Last Login",
                 'display' => function ($lastLogin) {
-                    return $lastLogin ? date('H:i - d M Y', $lastLogin) : 'Never';
+                    return $lastLogin ? date('d-m-Y H:i:s', $lastLogin) : 'Never';
                 },
             ],
             "salt" => [
